SystemOrganization addCategory: #'DiffEng-Tests'!
SystemOrganization addCategory: #'DiffEng-Matching'!
SystemOrganization addCategory: #'DiffEng-Trees'!
SystemOrganization addCategory: #'DiffEng-Annotation'!

!Collection methodsFor: '*diffeng-enumerating' stamp: 'cwp 6/20/2004 00:37'!
with: aCollection pairsDo: aBlock 
	^ self do: [:own | aCollection do: [:other | aBlock value: own value: other]]! !

TestCase subclass: #DEBipartiteGraphTest
	instanceVariableNames: 'graph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DiffEng-Tests'!

!DEBipartiteGraphTest methodsFor: 'testing' stamp: 'cwp 6/22/2004 17:02'!
testEdgeValidity
	self assert: self edges size = 11.
! !

!DEBipartiteGraphTest methodsFor: 'testing' stamp: 'cwp 6/22/2004 17:01'!
testCreationBlossom
	self assert: (self edges select: [:ea | ea source isCreation]) size = 3! !

!DEBipartiteGraphTest methodsFor: 'accessing' stamp: 'cwp 6/22/2004 17:03'!
edges
	^ graph edges! !

!DEBipartiteGraphTest methodsFor: 'testing' stamp: 'cwp 6/22/2004 17:01'!
testDestructionBlossom
	self assert: (self edges select: [:ea | ea target isDestruction]) size = 3! !

!DEBipartiteGraphTest methodsFor: 'testing' stamp: 'cwp 6/22/2004 17:03'!
setUp
	| tree |
	tree := DESourceResource current tree.
	graph := DEBipartiteGraph 
				source: tree
				target: tree
				comparator: [:s :t | t node couldMatch: s node].
! !

TestCase subclass: #DEAnnotationTest
	instanceVariableNames: 'graph edges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DiffEng-Tests'!

!DEAnnotationTest methodsFor: 'asserting' stamp: 'cwp 6/22/2004 13:57'!
assertFirstEdgeIsPruned
	self assert: edges first annotation operation = #prune.
	self assert: edges first annotation updateOps isEmpty.
	self assert: edges first annotation dependency = nil! !

!DEAnnotationTest methodsFor: 'tests' stamp: 'cwp 6/22/2004 15:43'!
testAttachTwig
	self source: #(a) target: #(a ((b (c)))).
	self assertEdgesTo: #b are: #(attach).
	self assertEdgesTo: #c are: #(noOp).! !

!DEAnnotationTest methodsFor: 'asserting' stamp: 'cwp 6/22/2004 15:40'!
assertEdgesTo: aSymbol are: anArray 
	| ops blossom |
	blossom _ (graph edges
				detect: [:ea | ea target hasNode and: [ea target node name = aSymbol]]
				ifNone: [self signalFailure: 'no target nodes named ', aSymbol]) target.
	ops _ blossom edges collect: [:ea | ea annotation operation].
	self assert: ops asArray = anArray.
			! !

!DEAnnotationTest methodsFor: 'tests' stamp: 'cwp 6/22/2004 16:58'!
testSimpleJoin
	self source:  #(a ((b) (b))) target:  #(a (b)).
	self assertEdgesTo: #b are: #(noOp join).! !

!DEAnnotationTest methodsFor: 'tests' stamp: 'cwp 6/22/2004 17:06'!
testJoinWithMove
	self source: #(a ((b) (d (c c)))) target: #(a ((b (c)))). 
	self assertEdgesTo: #c are: #(move join).! !

!DEAnnotationTest methodsFor: 'tests' stamp: 'cwp 6/22/2004 15:45'!
testPruneLeaf
	self source: #(a (b)) target: #(a).
	self assertEdgesFrom: #b are: #(prune).! !

!DEAnnotationTest methodsFor: 'tests' stamp: 'cwp 6/22/2004 15:47'!
testPruneTwig
	self source: #(a ((b (c)))) target:  #(a).
	self assertEdgesFrom: #b are: #(prune).
	self assertEdgesFrom: #c are: #(noOp).! !

!DEAnnotationTest methodsFor: 'graph building' stamp: 'cwp 6/22/2004 15:56'!
prune
	graph edges 
		removeAllSuchThat: [:ea | ea pruneIfTrue: ((ea isCreation and: [ea target isStem])
								  or: [ea isDestruction and: [ea source isStem]])].
	
	graph edges
		do: [:ea | ea source isStem ifTrue: [self assert: ea target isPetal].
				  ea target isStem ifTrue: [self assert: ea source isPetal]]
	! !

!DEAnnotationTest methodsFor: 'tests' stamp: 'cwp 6/22/2004 15:40'!
testAttachLeaf
	self source: #(a) target: #(a (b)).
	self assertEdgesTo: #b are: #(attach).! !

!DEAnnotationTest methodsFor: 'tests' stamp: 'cwp 6/22/2004 14:05'!
testCopyWithMove
	self source: #(a ((b (c)))) target: #(a ((b) (d (c c)))).
	self assertEdgesFrom: #c are: #(move copy).! !

!DEAnnotationTest methodsFor: 'graph building' stamp: 'cwp 6/22/2004 13:43'!
treeFromSExpression: sexpr
	| node |
	sexpr isSymbol ifTrue: [^ DENodeA name: sexpr]. 
	sexpr size = 1 ifTrue: [^ DENodeA name: sexpr first].
	sexpr size = 2 ifTrue: [node _ DENodeA name: sexpr first.
						sexpr second do: [:ea | node addLeft: (self treeFromSExpression: ea)].
						^ node].
	sexpr size = 3 ifTrue: [node _ DENodeA name: sexpr first.
						sexpr second do: [:ea | node addLeft: (self treeFromSExpression: ea)].
						sexpr third do: [:ea | node addRight: (self treeFromSExpression: ea)].
						^ node].
	sexpr size = 4 ifTrue: [node _ DENodeA name: sexpr first.
						node value: sexpr second.
						sexpr third do: [:ea | node addLeft: (self treeFromSExpression: ea)].
						sexpr fourth do: [:ea | node addRight: (self treeFromSExpression: ea)].
						^ node].
		! !

!DEAnnotationTest methodsFor: 'tests' stamp: 'cwp 6/22/2004 15:50'!
testSimpleCopy
	self source: #(a (b)) target: #(a ((b) (b))). 
	self assertEdgesFrom: #b are: #(noOp copy).! !

!DEAnnotationTest methodsFor: 'asserting' stamp: 'cwp 6/22/2004 15:36'!
assertEdgesFrom: aSymbol are: anArray 
	| source ops |
	source _ (graph edges
				detect: [:ea | ea source hasNode and: [ea source node name = aSymbol]]
				ifNone: [self signalFailure: 'no source nodes named ', aSymbol]) source.
	ops _ source edges collect: [:ea | ea annotation operation].
	self assert: ops asArray = anArray.
			! !

!DEAnnotationTest methodsFor: 'asserting' stamp: 'cwp 6/22/2004 13:57'!
assertEdgeAt: index is: operation 
	| annotation |
	annotation := (edges at: index) annotation.
	self assert: annotation operation = operation.
	self assert: annotation updateOps isEmpty.
	self assert: annotation dependency = nil! !

!DEAnnotationTest methodsFor: 'graph building' stamp: 'cwp 6/22/2004 14:44'!
source: sexpr1 target: sexpr2
	| sTree tTree |
	sTree _ self treeFromSExpression: sexpr1.
	tTree _ self treeFromSExpression: sexpr2.
	graph _ DEBipartiteGraph 
				source: sTree 
				target: tTree 
				comparator: [:s :t | s node name = t node name].
	self prune.
	DEAnnotator annotate: graph! !

!DEAnnotationTest methodsFor: 'tests' stamp: 'cwp 6/22/2004 17:33'!
testSimpleMove
	self source: #(a ((b (c)) (d))) target: #(a ((b) (d (c)))).
	self assertEdgesTo: #c are: #(move)! !

Object subclass: #DEBipartiteGraph
	instanceVariableNames: 'edges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DiffEng-Matching'!

!DEBipartiteGraph class methodsFor: 'instance creation' stamp: 'cwp 6/22/2004 14:18'!
source: sourceTree target: targetTree comparator: comparator
	| source target edges |
	source _ sourceTree allNodes collect: [:ea | DENodeBlossom for: ea].
	target _ targetTree allNodes collect: [:ea | DENodeBlossom for: ea].
	source add: DECreationBlossom new.
	target add: DEDestructionBlossom new.
	edges _ OrderedCollection new.
	source with: target pairsDo: 
		[:s :t | (s couldMatch: t with: comparator) ifTrue: [edges add: (DEEdge source: s target: t)]].
	^ self edges: edges! !

!DEBipartiteGraph methodsFor: 'testing' stamp: 'cwp 6/21/2004 19:39'!
parentsMatch: edge
	| sParent tParent |
	sParent _ self parentOf: edge source.
	tParent _ self parentOf: edge target.
	(sParent isNil and: [tParent isNil]) ifTrue: [^ true].
	(sParent isNil or: [tParent isNil]) ifTrue: [^ false].
	^ sParent edges size = 1
		and: [tParent edges size = 1]
		and: [sParent edges first = tParent edges first]! !

!DEBipartiteGraph methodsFor: 'accessing' stamp: 'cwp 6/21/2004 12:55'!
edges: aCollection
	edges _ aCollection! !

!DEBipartiteGraph class methodsFor: 'instance creation' stamp: 'cwp 6/21/2004 12:56'!
edges: edges
	^ self new edges: edges! !

!DEBipartiteGraph methodsFor: 'accessing' stamp: 'cwp 6/20/2004 13:38'!
edges
	^ edges! !

!DEBipartiteGraph methodsFor: 'accessing' stamp: 'cwp 6/21/2004 16:38'!
parentOf: aBlossom
	| parent |
	(parent _ aBlossom node parent) ifNil: [^ nil].
	edges do: [:edge | edge nodeBlossomsDo: [:b | b node == parent ifTrue: [ ^ b]]].
	! !

Object subclass: #DENode
	instanceVariableNames: 'parent name label left right'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DiffEng-Trees'!

!DENode methodsFor: 'accessing' stamp: 'cwp 6/19/2004 22:13'!
left
	^ left ifNil: [#()]! !

!DENode methodsFor: 'accessing' stamp: 'cwp 6/19/2004 22:16'!
parent: anObject
	parent := anObject! !

!DENode methodsFor: 'accessing' stamp: 'cwp 6/19/2004 22:13'!
right
	^ right ifNil: [#()]! !

DENode subclass: #DENodeA
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DiffEng-Trees'!

DENode subclass: #DENodeB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DiffEng-Trees'!

!DENode methodsFor: 'testing' stamp: 'cwp 6/20/2004 00:10'!
couldMatch: aNode
	^ self class = aNode class! !

!DENode methodsFor: 'accessing' stamp: 'cwp 6/19/2004 22:15'!
addLeft: aNode
	left ifNil: [left _ OrderedCollection new].
	left add: aNode.
	aNode parent: self.
	^ aNode! !

!DENode methodsFor: 'accessing' stamp: 'cwp 6/19/2004 22:12'!
name
	^name! !

!DENode methodsFor: 'testing' stamp: 'cwp 6/21/2004 21:14'!
isBlossom
	^ false! !

!DENode class methodsFor: 'as yet unclassified' stamp: 'cwp 6/19/2004 23:37'!
name: aSymbol label: aSymbol2 
	^ self new name: aSymbol; label: aSymbol2! !

!DENode methodsFor: 'enumerating' stamp: 'cwp 6/20/2004 00:16'!
allNodes
	^ self addAllNodesTo: Set new
! !

!DENode class methodsFor: 'instance creation' stamp: 'cwp 6/22/2004 13:44'!
name: aSymbol 
	^ self new name: aSymbol! !

!DENode methodsFor: 'accessing' stamp: 'cwp 6/19/2004 22:16'!
parent
	^parent! !

!DENode methodsFor: 'enumerating' stamp: 'cwp 6/20/2004 00:19'!
addAllNodesTo: aCollection
	aCollection add: self.
	self left do: [:ea | ea addAllNodesTo: aCollection].
	self right do: [:ea | ea addAllNodesTo: aCollection].
	^ aCollection! !

!DENode methodsFor: 'accessing' stamp: 'cwp 6/19/2004 22:16'!
addRight: aNode
	right ifNil: [right _ SortedCollection new].
	right add: aNode.
	aNode parent: self.
	^ aNode! !

!DENode methodsFor: 'accessing' stamp: 'cwp 6/19/2004 22:12'!
label: anObject
	label := anObject! !

!DENode methodsFor: 'accessing' stamp: 'cwp 6/19/2004 22:12'!
label
	^label! !

!DENode methodsFor: 'accessing' stamp: 'cwp 6/19/2004 22:12'!
name: anObject
	name := anObject! !

Object subclass: #DEEdge
	instanceVariableNames: 'source target annotation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DiffEng-Matching'!

!DEEdge methodsFor: 'testing' stamp: 'cwp 6/21/2004 16:44'!
isDestruction
	^ target isDestruction! !

!DEEdge methodsFor: 'testing' stamp: 'cwp 6/21/2004 16:33'!
isCreation
	^ source isCreation! !

!DEEdge methodsFor: 'releasing' stamp: 'cwp 6/22/2004 14:43'!
prune
	source removeEdge: self.
	target removeEdge: self! !

!DEEdge methodsFor: 'enumerating' stamp: 'cwp 6/21/2004 16:33'!
nodeBlossomsDo: aBlock
	source isCreation ifFalse: [aBlock value: source].
	target isDestruction ifFalse: [aBlock value: target]! !

!DEEdge methodsFor: 'accessing' stamp: 'cwp 6/21/2004 12:04'!
update: aSymbol
	self annotation addUpdateOp: aSymbol! !

!DEEdge methodsFor: 'accessing' stamp: 'cwp 6/21/2004 12:03'!
annotation
	^ annotation ifNil: [annotation _ DEAnnotation noOp]! !

!DEEdge methodsFor: 'releasing' stamp: 'cwp 6/22/2004 14:43'!
pruneIfTrue: aBoolean 
	aBoolean ifTrue: [self prune].
	^aBoolean! !

!DEEdge methodsFor: 'accessing' stamp: 'cwp 6/21/2004 11:26'!
target: anObject
	target := anObject.
	target addEdge: self! !

!DEEdge methodsFor: 'accessing' stamp: 'cwp 6/20/2004 00:03'!
source
	^source! !

!DEEdge methodsFor: 'enumerating' stamp: 'cwp 6/21/2004 18:47'!
blossomsDo: aBlockContext 
	aBlockContext value: source value: target! !

!DEEdge methodsFor: 'accessing' stamp: 'cwp 6/22/2004 13:57'!
op: aSymbol
	self annotation operation: aSymbol! !

!DEEdge methodsFor: 'accessing' stamp: 'cwp 6/20/2004 00:03'!
target
	^target! !

!DEEdge methodsFor: 'printing' stamp: 'cwp 6/22/2004 14:37'!
printOn: stream
	stream nextPutAll: self class name.
	source printIdentityOn: stream.
	target printIdentityOn: stream.! !

!DEEdge methodsFor: 'accessing' stamp: 'cwp 6/21/2004 11:26'!
source: anObject
	source := anObject.
	source addEdge: self! !

!DEEdge methodsFor: 'testing' stamp: 'cwp 6/21/2004 19:34'!
isPair
	^ source edges size = 1 and: [target edges size = 1]! !

!DEEdge class methodsFor: 'instance creation' stamp: 'cwp 6/20/2004 00:39'!
source: aBlossom1 target: aBlossom2 
	^ self new source: aBlossom1; target: aBlossom2! !

Object subclass: #DEAnnotation
	instanceVariableNames: 'operation updateOps dependency'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DiffEng-Annotation'!

!DEAnnotation methodsFor: 'accessing' stamp: 'cwp 6/21/2004 11:43'!
dependency: anObject
	dependency := anObject! !

!DEAnnotation methodsFor: 'accessing' stamp: 'cwp 6/22/2004 13:56'!
operation
	^operation! !

!DEAnnotation class methodsFor: 'as yet unclassified' stamp: 'cwp 6/21/2004 12:40'!
noOp
	^ self op: #noOp! !

!DEAnnotation methodsFor: 'accessing' stamp: 'cwp 6/21/2004 11:49'!
updateOps
	^ updateOps ifNil: [Set new]! !

!DEAnnotation methodsFor: 'accessing' stamp: 'cwp 6/21/2004 11:44'!
addUpdateOp: aSelector
	updateOps ifNil: [updateOps _ Set new].
	updateOps add: aSelector! !

!DEAnnotation class methodsFor: 'as yet unclassified' stamp: 'cwp 6/22/2004 13:57'!
op: aStructuralOperation
	^ self new operation: aStructuralOperation! !

!DEAnnotation methodsFor: 'accessing' stamp: 'cwp 6/22/2004 13:56'!
operation: anObject
	operation := anObject! !

!DEAnnotation methodsFor: 'accessing' stamp: 'cwp 6/21/2004 11:43'!
dependency
	^dependency! !

Object subclass: #DEAnnotator
	instanceVariableNames: 'graph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DiffEng-Annotation'!

!DEAnnotator methodsFor: 'annotating' stamp: 'cwp 6/21/2004 12:51'!
annotate
	self edges do: [:ea | self annotate: ea]! !

!DEAnnotator methodsFor: 'accessing' stamp: 'cwp 6/21/2004 13:07'!
graph: anObject
	graph := anObject! !

!DEAnnotator methodsFor: 'annotating' stamp: 'cwp 6/21/2004 16:40'!
attach: edge 
	| parent |
	parent _ graph parentOf: edge target.
	(parent notNil and: [parent edges first isCreation])
		ifTrue: [edge op: #noOp]
		ifFalse: [edge op: #attach]! !

!DEAnnotator methodsFor: 'annotating' stamp: 'cwp 6/21/2004 16:43'!
prune: edge 
	| parent |
	parent _ graph parentOf: edge source.
	(parent notNil and: [parent edges first isDestruction])
		ifTrue: [edge op: #noOp]
		ifFalse: [edge op: #prune]! !

!DEAnnotator methodsFor: 'accessing' stamp: 'cwp 6/21/2004 13:07'!
graph
	^graph! !

!DEAnnotator methodsFor: 'annotating' stamp: 'cwp 6/22/2004 16:57'!
flower: blossom operation: opName
	| distinguishedEdge op |
	distinguishedEdge _ blossom edges detect: [:e | graph parentsMatch: e] ifNone: [].
	op _ distinguishedEdge 
		ifNil: [distinguishedEdge _ blossom edges first. #move]
		ifNotNil: [#noOp].
	distinguishedEdge op: op.
	blossom edges do: [:ea | ea op: opName] without: distinguishedEdge.! !

!DEAnnotator methodsFor: 'annotating' stamp: 'cwp 6/21/2004 19:34'!
pair: edge
	edge op: ((graph parentsMatch: edge) ifTrue: [#noOp] ifFalse: [#move]).! !

!DEAnnotator methodsFor: 'annotating' stamp: 'cwp 6/22/2004 16:57'!
annotate: edge 
	edge isCreation ifTrue: [^ self attach: edge].
	edge isDestruction ifTrue: [^ self prune: edge].
	edge isPair ifTrue: [^ self pair: edge].
	edge source isStem ifTrue: [^ self flower: edge source operation: #copy].
	edge target isStem ifTrue: [^ self flower: edge target operation: #join].! !

!DEAnnotator class methodsFor: 'as yet unclassified' stamp: 'cwp 6/21/2004 12:53'!
annotate: aGraph 
	^ (self forGraph: aGraph) annotate! !

!DEAnnotator methodsFor: 'accessing' stamp: 'cwp 6/21/2004 13:15'!
edges
	^ graph edges! !

!DEAnnotator class methodsFor: 'as yet unclassified' stamp: 'cwp 6/21/2004 12:53'!
forGraph: aBipartiteGraph
	^ self new graph: aBipartiteGraph! !

Object subclass: #DEBlossom
	instanceVariableNames: 'edges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DiffEng-Matching'!

!DEBlossom methodsFor: 'testing' stamp: 'cwp 6/21/2004 18:48'!
isPetal
	^ edges size = 1! !

!DEBlossom methodsFor: 'printing' stamp: 'cwp 6/22/2004 14:36'!
printOn: stream 
	stream nextPutAll: self class name.
	self printIdentityOn: stream! !

!DEBlossom methodsFor: 'printing' stamp: 'cwp 6/22/2004 14:36'!
printIdentityOn: stream 
	stream nextPut: $<.
	self identityHash printOn: stream.
	stream nextPut: $-.
	edges size printOn: stream.
	stream nextPut: $>! !

!DEBlossom methodsFor: 'accessing' stamp: 'cwp 6/22/2004 14:41'!
removeEdge: anEdge
	edges remove: anEdge! !

DEBlossom subclass: #DEDestructionBlossom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DiffEng-Matching'!

!DEDestructionBlossom methodsFor: 'testing' stamp: 'cwp 6/20/2004 12:53'!
isDestruction
	^ true! !

!DEDestructionBlossom methodsFor: 'testing' stamp: 'cwp 6/22/2004 14:21'!
couldMatch: aBlossom with: comparator
	^ aBlossom isCreation not! !

!DEBlossom methodsFor: 'accessing' stamp: 'cwp 6/21/2004 16:34'!
addEdge: anEdge
	edges ifNil: [edges _ OrderedCollection new].
	edges add: anEdge! !

!DEBlossom methodsFor: 'testing' stamp: 'cwp 6/20/2004 12:53'!
isDestruction
	^ false! !

!DEBlossom methodsFor: 'testing' stamp: 'cwp 6/21/2004 18:48'!
isStem
	^ edges size > 1! !

!DEBlossom methodsFor: 'testing' stamp: 'cwp 6/21/2004 21:18'!
isBlossom
	^ true! !

!DEBlossom methodsFor: 'testing' stamp: 'cwp 6/20/2004 12:52'!
isCreation
	^ false! !

!DEBlossom methodsFor: 'testing' stamp: 'cwp 6/20/2004 13:02'!
hasNode
	^ false! !

DEBlossom subclass: #DECreationBlossom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DiffEng-Matching'!

!DECreationBlossom methodsFor: 'testing' stamp: 'cwp 6/22/2004 14:20'!
couldMatch: aBlossom with: comparator
	^ aBlossom isDestruction not! !

!DECreationBlossom methodsFor: 'testing' stamp: 'cwp 6/20/2004 12:52'!
isCreation
	^ true! !

DEBlossom subclass: #DENodeBlossom
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DiffEng-Matching'!

!DENodeBlossom methodsFor: 'testing' stamp: 'cwp 6/22/2004 14:23'!
couldMatch: aBlossom with: comparator
	^ aBlossom hasNode
		ifFalse: [true]
		ifTrue: [comparator value: self value: aBlossom]! !

!DENodeBlossom methodsFor: 'testing' stamp: 'cwp 6/20/2004 13:02'!
hasNode
	^ true! !

!DENodeBlossom class methodsFor: 'as yet unclassified' stamp: 'cwp 6/19/2004 23:46'!
for: aNode
	^ self new node: aNode! !

!DENodeBlossom methodsFor: 'accessing' stamp: 'cwp 6/19/2004 23:46'!
node: aNode
	node _ aNode! !

!DENodeBlossom methodsFor: 'accessing' stamp: 'cwp 6/20/2004 00:08'!
node
	^ node! !

!DEBlossom methodsFor: 'accessing' stamp: 'cwp 6/20/2004 12:52'!
edges
	^ edges ifNil: [#()]! !

TestResource subclass: #DESourceResource
	instanceVariableNames: 'tree'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DiffEng-Tests'!

!DESourceResource methodsFor: 'accessing' stamp: 'cwp 6/19/2004 22:52'!
setUp
	tree _ DENodeA name: #alpha label: #first.
	tree 
		addLeft: (DENodeB name: #beta label: #second);
		addRight: (DENodeA name: #gamma label: #third)
! !

!DESourceResource methodsFor: 'accessing' stamp: 'cwp 6/19/2004 22:55'!
tree
	^ tree! !

TestResource subclass: #DETargetResource
	instanceVariableNames: 'tree'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DiffEng-Tests'!

!DETargetResource methodsFor: 'as yet unclassified' stamp: 'cwp 6/19/2004 23:06'!
tree
	^ tree! !

!DETargetResource methodsFor: 'as yet unclassified' stamp: 'cwp 6/19/2004 23:06'!
setUp
	tree _ DESourceResource current tree copy.
	! !
